"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CleanCommand = void 0;
const _ = require("lodash");
const yok_1 = require("../common/yok");
class CleanCommand {
    constructor($errors, $options, $platformCommandHelper, $platformValidationService, $platformEnvironmentRequirements, $projectData) {
        this.$errors = $errors;
        this.$options = $options;
        this.$platformCommandHelper = $platformCommandHelper;
        this.$platformValidationService = $platformValidationService;
        this.$platformEnvironmentRequirements = $platformEnvironmentRequirements;
        this.$projectData = $projectData;
        this.allowedParameters = [];
        this.$projectData.initializeProjectData();
    }
    execute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.$platformCommandHelper.cleanPlatforms(args, this.$projectData, this.$options.frameworkPath);
        });
    }
    canExecute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!args || args.length === 0) {
                this.$errors.failWithHelp("No platform specified. Please specify a platform to clean.");
            }
            _.each(args, (platform) => {
                this.$platformValidationService.validatePlatform(platform, this.$projectData);
            });
            for (const platform of args) {
                this.$platformValidationService.validatePlatformInstalled(platform, this.$projectData);
                const currentRuntimeVersion = this.$platformCommandHelper.getCurrentPlatformVersion(platform, this.$projectData);
                yield this.$platformEnvironmentRequirements.checkEnvironmentRequirements({
                    platform,
                    projectDir: this.$projectData.projectDir,
                    runtimeVersion: currentRuntimeVersion,
                    options: this.$options,
                });
            }
            return true;
        });
    }
}
exports.CleanCommand = CleanCommand;
yok_1.injector.registerCommand("platform|clean", CleanCommand);
//# sourceMappingURL=platform-clean.js.map