"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PreUninstallCommand = void 0;
const path = require("path");
const helpers_1 = require("../helpers");
const constants_1 = require("../../constants");
const yok_1 = require("../yok");
class PreUninstallCommand {
    constructor($analyticsService, $extensibilityService, $fs, 
    // private $opener: IOpener,
    $packageInstallationManager, $settingsService) {
        this.$analyticsService = $analyticsService;
        this.$extensibilityService = $extensibilityService;
        this.$fs = $fs;
        this.$packageInstallationManager = $packageInstallationManager;
        this.$settingsService = $settingsService;
        // disabled for now (6/24/2020)
        // private static FEEDBACK_FORM_URL = "https://www.nativescript.org/uninstall-feedback";
        this.allowedParameters = [];
    }
    execute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const isIntentionalUninstall = (0, helpers_1.doesCurrentNpmCommandMatch)([
                /^uninstall$/,
                /^remove$/,
                /^rm$/,
                /^r$/,
                /^un$/,
                /^unlink$/,
            ]);
            yield this.$analyticsService.trackEventActionInGoogleAnalytics({
                action: "Uninstall CLI" /* TrackActionNames.UninstallCLI */,
                additionalData: `isIntentionalUninstall${constants_1.AnalyticsEventLabelDelimiter}${isIntentionalUninstall}${constants_1.AnalyticsEventLabelDelimiter}isInteractive${constants_1.AnalyticsEventLabelDelimiter}${!!(0, helpers_1.isInteractive)()}`,
            });
            if (isIntentionalUninstall) {
                yield this.handleIntentionalUninstall();
            }
            this.$fs.deleteFile(path.join(this.$settingsService.getProfileDir(), "KillSwitches", "cli"));
            yield this.$analyticsService.finishTracking();
        });
    }
    handleIntentionalUninstall() {
        return __awaiter(this, void 0, void 0, function* () {
            this.$extensibilityService.removeAllExtensions();
            this.$packageInstallationManager.clearInspectorCache();
            yield this.handleFeedbackForm();
        });
    }
    handleFeedbackForm() {
        return __awaiter(this, void 0, void 0, function* () {
            // disabled for now (6/24/2020)
            // if (isInteractive()) {
            // 	this.$opener.open(PreUninstallCommand.FEEDBACK_FORM_URL);
            // }
            return Promise.resolve();
        });
    }
}
exports.PreUninstallCommand = PreUninstallCommand;
yok_1.injector.registerCommand("dev-preuninstall", PreUninstallCommand);
//# sourceMappingURL=preuninstall.js.map